@@ -1,11 +1,23 @@
 require 'rubygems'
 require 'minitest/autorun'
 require 'net/http/persistent'
-require 'openssl'
 require 'stringio'
 
-class Net::HTTP::Persistent::SSLReuse
-  alias orig_connect connect
+HAVE_OPENSSL = defined?(OpenSSL::SSL)
+
+module Net::HTTP::Persistent::TestConnect
+  def self.included mod
+    mod.send :alias_method, :orig_connect, :connect
+
+    def mod.use_connect which
+      self.send :remove_method, :connect
+      self.send :alias_method, :connect, which
+    end
+  end
+
+  def host_down_connect
+    raise Errno::EHOSTDOWN
+  end
 
   def test_connect
     unless use_ssl? then
@@ -30,16 +42,32 @@ class Net::HTTP::Persistent::SSLReuse
     @socket = Net::BufferedIO.new s
   end
 
-  def self.use_connect which
-    self.send :remove_method, :connect
-    self.send :alias_method, :connect, which
+  def refused_connect
+    raise Errno::ECONNREFUSED
   end
 end
 
-class TestNetHttpPersistent < MiniTest::Unit::TestCase
+class Net::HTTP
+  include Net::HTTP::Persistent::TestConnect
+end
+
+class Net::HTTP::Persistent::SSLReuse
+  include Net::HTTP::Persistent::TestConnect
+end
+
+class TestNetHttpPersistent < Minitest::Test
+
+  RUBY_1 = RUBY_VERSION < '2'
 
   def setup
+    @http_class = if RUBY_1 and HAVE_OPENSSL then
+                    Net::HTTP::Persistent::SSLReuse
+                  else
+                    Net::HTTP
+                  end
+
     @http = Net::HTTP::Persistent.new
+
     @uri  = URI.parse 'http://example.com/path'
 
     ENV.delete 'http_proxy'
@@ -48,7 +76,10 @@ class TestNetHttpPersistent < MiniTest::Unit::TestCase
     ENV.delete 'HTTP_PROXY_USER'
     ENV.delete 'http_proxy_pass'
     ENV.delete 'HTTP_PROXY_PASS'
+    ENV.delete 'no_proxy'
+    ENV.delete 'NO_PROXY'
 
+    Net::HTTP.use_connect :test_connect
     Net::HTTP::Persistent::SSLReuse.use_connect :test_connect
   end
 
@@ -57,12 +88,14 @@ class TestNetHttpPersistent < MiniTest::Unit::TestCase
       Thread.current[key] = nil
     end
 
+    Net::HTTP.use_connect :orig_connect
     Net::HTTP::Persistent::SSLReuse.use_connect :orig_connect
   end
 
   class BasicConnection
-    attr_accessor :started, :finished, :address, :port
-    attr_reader :req
+    attr_accessor :started, :finished, :address, :port,
+                  :read_timeout, :open_timeout
+    attr_reader :req, :debug_output
     def initialize
       @started, @finished = 0, 0
       @address, @port = 'example.com', 80
@@ -80,6 +113,9 @@ class TestNetHttpPersistent < MiniTest::Unit::TestCase
     def reset?
       @started == @finished + 1
     end
+    def set_debug_output io
+      @debug_output = io
+    end
     def start
       @started += 1
       io = Object.new
@@ -142,6 +178,10 @@ class TestNetHttpPersistent < MiniTest::Unit::TestCase
   def test_initialize
     assert_nil @http.proxy_uri
 
+    assert_empty @http.no_proxy
+
+    skip 'OpenSSL is missing' unless HAVE_OPENSSL
+
     ssl_session_exists = OpenSSL::SSL.const_defined? :Session
 
     assert_equal ssl_session_exists, @http.reuse_ssl_sessions
@@ -153,6 +193,8 @@ class TestNetHttpPersistent < MiniTest::Unit::TestCase
   end
 
   def test_initialize_no_ssl_session
+    skip 'OpenSSL is missing' unless HAVE_OPENSSL
+
     skip "OpenSSL::SSL::Session does not exist on #{RUBY_PLATFORM}" unless
       OpenSSL::SSL.const_defined? :Session
 
@@ -182,6 +224,39 @@ class TestNetHttpPersistent < MiniTest::Unit::TestCase
     assert_equal 1, @http.ssl_generation
   end
 
+  def test_can_retry_eh_change_requests
+    post = Net::HTTP::Post.new '/'
+
+    refute @http.can_retry? post
+
+    @http.retry_change_requests = true
+
+    assert @http.can_retry? post
+  end
+
+  if RUBY_1 then
+    def test_can_retry_eh_idempotent
+      head = Net::HTTP::Head.new '/'
+
+      assert @http.can_retry? head
+
+      post = Net::HTTP::Post.new '/'
+
+      refute @http.can_retry? post
+    end
+  else
+    def test_can_retry_eh_idempotent
+      head = Net::HTTP::Head.new '/'
+
+      assert @http.can_retry? head
+      refute @http.can_retry? head, true
+
+      post = Net::HTTP::Post.new '/'
+
+      refute @http.can_retry? post
+    end
+  end
+
   def test_cert_store_equals
     @http.cert_store = :cert_store
 
@@ -201,7 +276,7 @@ class TestNetHttpPersistent < MiniTest::Unit::TestCase
     @http.read_timeout = 321
     c = @http.connection_for @uri
 
-    assert_kind_of Net::HTTP::Persistent::SSLReuse, c
+    assert_kind_of @http_class, c
 
     assert c.started?
     refute c.proxy?
@@ -211,15 +286,6 @@ class TestNetHttpPersistent < MiniTest::Unit::TestCase
 
     assert_includes conns[0].keys, 'example.com:80'
     assert_same c, conns[0]['example.com:80']
-
-    socket = c.instance_variable_get :@socket
-    expected = if Socket.const_defined? :TCP_NODELAY then
-                 [[Socket::IPPROTO_TCP, Socket::TCP_NODELAY, 1]]
-               else
-                 []
-               end
-
-    assert_equal expected, socket.io.instance_variable_get(:@setsockopts)
   end
 
   def test_connection_for_cached
@@ -227,10 +293,14 @@ class TestNetHttpPersistent < MiniTest::Unit::TestCase
     cached.start
     conns[0]['example.com:80'] = cached
 
+    @http.read_timeout = 5
+
     c = @http.connection_for @uri
 
     assert c.started?
 
+    assert_equal 5,       c.read_timeout
+
     assert_same cached, c
   end
 
@@ -324,7 +394,20 @@ class TestNetHttpPersistent < MiniTest::Unit::TestCase
                  'connection not reset due to timeout'
   end
 
+  def test_connection_for_refused
+    Net::HTTP.use_connect :refused_connect
+    Net::HTTP::Persistent::SSLReuse.use_connect :refused_connect
+
+    e = assert_raises Net::HTTP::Persistent::Error do
+      @http.connection_for @uri
+    end
+
+    assert_equal 'connection refused: example.com:80', e.message
+  end
+
   def test_connection_for_finished_ssl
+    skip 'OpenSSL is missing' unless HAVE_OPENSSL
+
     uri = URI.parse 'https://example.com/path'
     c = @http.connection_for uri
 
@@ -350,7 +433,7 @@ class TestNetHttpPersistent < MiniTest::Unit::TestCase
       @http.connection_for @uri
     end
 
-    assert_match %r%host down%, e.message
+    assert_equal 'host down: example.com:80', e.message
   end
 
   def test_connection_for_http_class_with_fakeweb
@@ -373,6 +456,16 @@ class TestNetHttpPersistent < MiniTest::Unit::TestCase
     end
   end
 
+  def test_connection_for_http_class_with_artifice
+    Object.send :const_set, :Artifice, nil
+    c = @http.connection_for @uri
+    assert_instance_of Net::HTTP, c
+  ensure
+    if Object.const_defined?(:Artifice) then
+      Object.send :remove_const, :Artifice
+    end
+  end
+
   def test_connection_for_name
     http = Net::HTTP::Persistent.new 'name'
     uri = URI.parse 'http://example/'
@@ -410,6 +503,57 @@ class TestNetHttpPersistent < MiniTest::Unit::TestCase
     assert_same c, conns[1]['example.com:80:proxy.example:80:johndoe:muffins']
   end
 
+  def test_connection_for_proxy_host_down
+    Net::HTTP.use_connect :host_down_connect
+    Net::HTTP::Persistent::SSLReuse.use_connect :host_down_connect
+
+    uri = URI.parse 'http://proxy.example'
+    uri.user     = 'johndoe'
+    uri.password = 'muffins'
+
+    http = Net::HTTP::Persistent.new nil, uri
+
+    e = assert_raises Net::HTTP::Persistent::Error do
+      http.connection_for @uri
+    end
+
+    assert_equal 'host down: proxy.example:80', e.message
+  end
+
+  def test_connection_for_proxy_refused
+    Net::HTTP.use_connect :refused_connect
+    Net::HTTP::Persistent::SSLReuse.use_connect :refused_connect
+
+    uri = URI.parse 'http://proxy.example'
+    uri.user     = 'johndoe'
+    uri.password = 'muffins'
+
+    http = Net::HTTP::Persistent.new nil, uri
+
+    e = assert_raises Net::HTTP::Persistent::Error do
+      http.connection_for @uri
+    end
+
+    assert_equal 'connection refused: proxy.example:80', e.message
+  end
+
+  def test_connection_for_no_proxy
+    uri = URI.parse 'http://proxy.example'
+    uri.user     = 'johndoe'
+    uri.password = 'muffins'
+    uri.query    = 'no_proxy=example.com'
+
+    http = Net::HTTP::Persistent.new nil, uri
+
+    c = http.connection_for @uri
+
+    assert c.started?
+    refute c.proxy?
+
+    assert_includes conns[1].keys, 'example.com:80'
+    assert_same c, conns[1]['example.com:80']
+  end
+
   def test_connection_for_refused
     cached = basic_connection
     def cached.start; raise Errno::ECONNREFUSED end
@@ -423,21 +567,9 @@ class TestNetHttpPersistent < MiniTest::Unit::TestCase
     assert_match %r%connection refused%, e.message
   end
 
-  def test_connection_for_socket_options
-    @http.socket_options << [Socket::SOL_SOCKET, Socket::SO_KEEPALIVE, 1]
-    c = @http.connection_for @uri
-
-    socket = c.instance_variable_get :@socket
-
-    expected = []
-    expected << [Socket::IPPROTO_TCP, Socket::TCP_NODELAY, 1] if
-      Socket.const_defined? :TCP_NODELAY
-    expected << [Socket::SOL_SOCKET, Socket::SO_KEEPALIVE, 1]
-
-    assert_equal expected, socket.io.instance_variable_get(:@setsockopts)
-  end
-
   def test_connection_for_ssl
+    skip 'OpenSSL is missing' unless HAVE_OPENSSL
+
     uri = URI.parse 'https://example.com/path'
     c = @http.connection_for uri
 
@@ -446,6 +578,8 @@ class TestNetHttpPersistent < MiniTest::Unit::TestCase
   end
 
   def test_connection_for_ssl_cached
+    skip 'OpenSSL is missing' unless HAVE_OPENSSL
+
     @uri = URI.parse 'https://example.com/path'
 
     cached = ssl_connection 0
@@ -456,6 +590,8 @@ class TestNetHttpPersistent < MiniTest::Unit::TestCase
   end
 
   def test_connection_for_ssl_cached_reconnect
+    skip 'OpenSSL is missing' unless HAVE_OPENSSL
+
     @uri = URI.parse 'https://example.com/path'
 
     cached = ssl_connection
@@ -468,6 +604,8 @@ class TestNetHttpPersistent < MiniTest::Unit::TestCase
   end
 
   def test_connection_for_ssl_case
+    skip 'OpenSSL is missing' unless HAVE_OPENSSL
+
     uri = URI.parse 'HTTPS://example.com/path'
     c = @http.connection_for uri
 
@@ -508,6 +646,7 @@ class TestNetHttpPersistent < MiniTest::Unit::TestCase
 
   def test_expired_eh
     c = basic_connection
+    reqs[c.object_id] = 0
     touts[c.object_id] = Time.now - 11
 
     @http.idle_timeout = 0
@@ -526,6 +665,23 @@ class TestNetHttpPersistent < MiniTest::Unit::TestCase
     refute @http.expired? c
   end
 
+  def test_expired_due_to_max_requests
+    c = basic_connection
+    reqs[c.object_id] = 0
+    touts[c.object_id] = Time.now
+
+    refute @http.expired? c
+
+    reqs[c.object_id] = 10
+    refute @http.expired? c
+
+    @http.max_requests = 10
+    assert @http.expired? c
+
+    reqs[c.object_id] = 9
+    refute @http.expired? c
+  end
+
   def test_finish
     c = basic_connection
     reqs[c.object_id] = 5
@@ -620,7 +776,7 @@ class TestNetHttpPersistent < MiniTest::Unit::TestCase
   end
 
   def test_proxy_equals_env
-    ENV['HTTP_PROXY'] = 'proxy.example'
+    ENV['http_proxy'] = 'proxy.example'
 
     @http.proxy = :ENV
 
@@ -639,15 +795,17 @@ class TestNetHttpPersistent < MiniTest::Unit::TestCase
   end
 
   def test_proxy_from_env
-    ENV['HTTP_PROXY']      = 'proxy.example'
-    ENV['HTTP_PROXY_USER'] = 'johndoe'
-    ENV['HTTP_PROXY_PASS'] = 'muffins'
+    ENV['http_proxy']      = 'proxy.example'
+    ENV['http_proxy_user'] = 'johndoe'
+    ENV['http_proxy_pass'] = 'muffins'
+    ENV['NO_PROXY']        = 'localhost,example.com'
 
     uri = @http.proxy_from_env
 
     expected = URI.parse 'http://proxy.example'
     expected.user     = 'johndoe'
     expected.password = 'muffins'
+    expected.query    = 'no_proxy=localhost%2Cexample.com'
 
     assert_equal expected, uri
   end
@@ -656,12 +814,14 @@ class TestNetHttpPersistent < MiniTest::Unit::TestCase
     ENV['http_proxy']      = 'proxy.example'
     ENV['http_proxy_user'] = 'johndoe'
     ENV['http_proxy_pass'] = 'muffins'
+    ENV['no_proxy']        = 'localhost,example.com'
 
     uri = @http.proxy_from_env
 
     expected = URI.parse 'http://proxy.example'
     expected.user     = 'johndoe'
     expected.password = 'muffins'
+    expected.query    = 'no_proxy=localhost%2Cexample.com'
 
     assert_equal expected, uri
   end
@@ -671,13 +831,73 @@ class TestNetHttpPersistent < MiniTest::Unit::TestCase
 
     assert_nil uri
 
-    ENV['HTTP_PROXY'] = ''
+    ENV['http_proxy'] = ''
 
     uri = @http.proxy_from_env
 
     assert_nil uri
   end
 
+  def test_proxy_from_env_no_proxy_star
+    uri = @http.proxy_from_env
+
+    assert_nil uri
+
+    ENV['http_proxy'] = 'proxy.example'
+    ENV['no_proxy'] = '*'
+
+    uri = @http.proxy_from_env
+
+    assert_nil uri
+  end
+
+  def test_proxy_bypass
+    ENV['http_proxy'] = 'proxy.example'
+    ENV['no_proxy'] = 'localhost,example.com:80'
+
+    @http.proxy = :ENV
+
+    assert @http.proxy_bypass? 'localhost', 80
+    assert @http.proxy_bypass? 'localhost', 443
+    assert @http.proxy_bypass? 'LOCALHOST', 80
+    assert @http.proxy_bypass? 'example.com', 80
+    refute @http.proxy_bypass? 'example.com', 443
+    assert @http.proxy_bypass? 'www.example.com', 80
+    refute @http.proxy_bypass? 'www.example.com', 443
+    assert @http.proxy_bypass? 'endingexample.com', 80
+    refute @http.proxy_bypass? 'example.org', 80
+  end
+
+  def test_proxy_bypass_space
+    ENV['http_proxy'] = 'proxy.example'
+    ENV['no_proxy'] = 'localhost, example.com'
+
+    @http.proxy = :ENV
+
+    assert @http.proxy_bypass? 'example.com', 80
+    refute @http.proxy_bypass? 'example.org', 80
+  end
+
+  def test_proxy_bypass_trailing
+    ENV['http_proxy'] = 'proxy.example'
+    ENV['no_proxy'] = 'localhost,example.com,'
+
+    @http.proxy = :ENV
+
+    assert @http.proxy_bypass? 'example.com', 80
+    refute @http.proxy_bypass? 'example.org', 80
+  end
+
+  def test_proxy_bypass_double_comma
+    ENV['http_proxy'] = 'proxy.example'
+    ENV['no_proxy'] = 'localhost,,example.com'
+
+    @http.proxy = :ENV
+
+    assert @http.proxy_bypass? 'example.com', 80
+    refute @http.proxy_bypass? 'example.org', 80
+  end
+
   def test_reconnect
     result = @http.reconnect
 
@@ -714,6 +934,18 @@ class TestNetHttpPersistent < MiniTest::Unit::TestCase
     assert_equal 1, reqs[c.object_id]
   end
 
+  def test_request_ETIMEDOUT
+    c = basic_connection
+    def c.request(*a) raise Errno::ETIMEDOUT, "timed out" end
+
+    e = assert_raises Net::HTTP::Persistent::Error do
+      @http.request @uri
+    end
+
+    assert_equal 0, reqs[c.object_id]
+    assert_match %r%too many connection resets%, e.message
+  end
+
   def test_request_bad_response
     c = basic_connection
     def c.request(*a) raise Net::HTTPBadResponse end
@@ -726,23 +958,38 @@ class TestNetHttpPersistent < MiniTest::Unit::TestCase
     assert_match %r%too many bad responses%, e.message
   end
 
-  def test_request_bad_response_retry
-    c = basic_connection
-    def c.request(*a)
-      if defined? @called then
-        r = Net::HTTPResponse.allocate
-        r.instance_variable_set :@header, {}
-        def r.http_version() '1.1' end
-        r
-      else
-        @called = true
+  if RUBY_1 then
+    def test_request_bad_response_retry
+      c = basic_connection
+      def c.request(*a)
+        if defined? @called then
+          r = Net::HTTPResponse.allocate
+          r.instance_variable_set :@header, {}
+          def r.http_version() '1.1' end
+          r
+        else
+          @called = true
+          raise Net::HTTPBadResponse
+        end
+      end
+
+      @http.request @uri
+
+      assert c.finished?
+    end
+  else
+    def test_request_bad_response_retry
+      c = basic_connection
+      def c.request(*a)
         raise Net::HTTPBadResponse
       end
+
+      assert_raises Net::HTTP::Persistent::Error do
+        @http.request @uri
+      end
+
+      assert c.finished?
     end
-
-    @http.request @uri
-
-    assert c.finished?
   end
 
   def test_request_bad_response_unsafe
@@ -871,6 +1118,18 @@ class TestNetHttpPersistent < MiniTest::Unit::TestCase
     assert c.finished?
   end
 
+  def test_request_exception
+    c = basic_connection
+    def c.request(*a) raise Exception, "very bad things happened" end
+
+    assert_raises Exception do
+      @http.request @uri
+    end
+
+    assert_equal 0, reqs[c.object_id]
+    assert c.finished?
+  end
+
   def test_request_invalid
     c = basic_connection
     def c.request(*a) raise Errno::EINVAL, "write" end
@@ -928,25 +1187,43 @@ class TestNetHttpPersistent < MiniTest::Unit::TestCase
     assert_match %r%too many connection resets%, e.message
   end
 
-  def test_request_reset_retry
-    c = basic_connection
-    touts[c.object_id] = Time.now
-    def c.request(*a)
-      if defined? @called then
-        r = Net::HTTPResponse.allocate
-        r.instance_variable_set :@header, {}
-        def r.http_version() '1.1' end
-        r
-      else
-        @called = true
+  if RUBY_1 then
+    def test_request_reset_retry
+      c = basic_connection
+      touts[c.object_id] = Time.now
+      def c.request(*a)
+        if defined? @called then
+          r = Net::HTTPResponse.allocate
+          r.instance_variable_set :@header, {}
+          def r.http_version() '1.1' end
+          r
+        else
+          @called = true
+          raise Errno::ECONNRESET
+        end
+      end
+
+      @http.request @uri
+
+      assert c.reset?
+      assert c.finished?
+    end
+  else
+    def test_request_reset_retry
+      c = basic_connection
+      touts[c.object_id] = Time.now
+
+      def c.request(*a)
         raise Errno::ECONNRESET
       end
+
+      assert_raises Net::HTTP::Persistent::Error do
+        @http.request @uri
+      end
+
+      refute c.reset?
+      assert c.finished?
     end
-
-    @http.request @uri
-
-    assert c.reset?
-    assert c.finished?
   end
 
   def test_request_reset_unsafe
@@ -969,6 +1246,8 @@ class TestNetHttpPersistent < MiniTest::Unit::TestCase
   end
 
   def test_request_ssl_error
+    skip 'OpenSSL is missing' unless HAVE_OPENSSL
+
     uri = URI.parse 'https://example.com/path'
     c = @http.connection_for uri
     def c.request(*)
@@ -981,6 +1260,57 @@ class TestNetHttpPersistent < MiniTest::Unit::TestCase
     assert_match %r%bad write retry%, e.message
   end
 
+  def test_request_setup
+    @http.override_headers['user-agent'] = 'test ua'
+    @http.headers['accept'] = 'text/*'
+
+    input = Net::HTTP::Post.new '/path'
+
+    req = @http.request_setup input
+
+    assert_same input,         req
+    assert_equal '/path',      req.path
+
+    assert_equal 'test ua',    req['user-agent']
+    assert_match %r%text/\*%,  req['accept']
+
+    assert_equal 'keep-alive', req['connection']
+    assert_equal '30',         req['keep-alive']
+  end
+
+  def test_request_setup_uri
+    uri = @uri + '?a=b'
+
+    req = @http.request_setup uri
+
+    assert_kind_of Net::HTTP::Get, req
+    assert_equal '/path?a=b',  req.path
+  end
+
+  def test_request_failed
+    c = basic_connection
+    reqs[c.object_id] = 1
+    touts[c.object_id] = Time.now
+
+    original = nil
+
+    begin
+      raise 'original'
+    rescue => original
+    end
+
+    req = Net::HTTP::Get.new '/'
+      
+    e = assert_raises Net::HTTP::Persistent::Error do
+      @http.request_failed original, req, c
+    end
+
+    assert_match "too many connection resets (due to original - RuntimeError)",
+                 e.message
+
+    assert_equal original.backtrace, e.backtrace
+  end
+
   def test_reset
     c = basic_connection
     c.start
@@ -1039,14 +1369,25 @@ class TestNetHttpPersistent < MiniTest::Unit::TestCase
 
     refute @http.retry_change_requests
 
-    assert @http.can_retry?(get)
+    if RUBY_1 then
+      assert @http.can_retry?(get)
+    else
+      assert @http.can_retry?(get)
+    end
+    refute @http.can_retry?(get, true)
     refute @http.can_retry?(post)
 
     @http.retry_change_requests = true
 
     assert @http.retry_change_requests
 
-    assert @http.can_retry?(get)
+    if RUBY_1 then
+      assert @http.can_retry?(get)
+    else
+      assert @http.can_retry?(get)
+      refute @http.can_retry?(get, true)
+    end
+
     assert @http.can_retry?(post)
   end
 
@@ -1113,13 +1454,10 @@ class TestNetHttpPersistent < MiniTest::Unit::TestCase
   end
 
   def test_shutdown_no_connections
-    conns
-    ssl_conns
-
     @http.shutdown
 
-    assert_empty Thread.current[@http.generation_key]
-    assert_empty Thread.current[@http.ssl_generation_key]
+    assert_nil Thread.current[@http.generation_key]
+    assert_nil Thread.current[@http.ssl_generation_key]
 
     assert_nil Thread.current[@http.request_key]
     assert_nil Thread.current[@http.timeout_key]
@@ -1143,6 +1481,8 @@ class TestNetHttpPersistent < MiniTest::Unit::TestCase
   end
 
   def test_shutdown_ssl
+    skip 'OpenSSL is missing' unless HAVE_OPENSSL
+
     @uri = URI 'https://example'
 
     @http.connection_for @uri
@@ -1182,6 +1522,8 @@ class TestNetHttpPersistent < MiniTest::Unit::TestCase
   end
 
   def test_ssl
+    skip 'OpenSSL is missing' unless HAVE_OPENSSL
+
     @http.verify_callback = :callback
     c = Net::HTTP.new 'localhost', 80
 
@@ -1194,6 +1536,8 @@ class TestNetHttpPersistent < MiniTest::Unit::TestCase
   end
 
   def test_ssl_ca_file
+    skip 'OpenSSL is missing' unless HAVE_OPENSSL
+
     @http.ca_file = 'ca_file'
     @http.verify_callback = :callback
     c = Net::HTTP.new 'localhost', 80
@@ -1206,6 +1550,8 @@ class TestNetHttpPersistent < MiniTest::Unit::TestCase
   end
 
   def test_ssl_cert_store
+    skip 'OpenSSL is missing' unless HAVE_OPENSSL
+
     store = OpenSSL::X509::Store.new
     @http.cert_store = store
 
@@ -1218,6 +1564,8 @@ class TestNetHttpPersistent < MiniTest::Unit::TestCase
   end
 
   def test_ssl_cert_store_default
+    skip 'OpenSSL is missing' unless HAVE_OPENSSL
+
     @http.verify_mode = OpenSSL::SSL::VERIFY_PEER
 
     c = Net::HTTP.new 'localhost', 80
@@ -1229,6 +1577,8 @@ class TestNetHttpPersistent < MiniTest::Unit::TestCase
   end
 
   def test_ssl_certificate
+    skip 'OpenSSL is missing' unless HAVE_OPENSSL
+
     @http.certificate = :cert
     @http.private_key = :key
     c = Net::HTTP.new 'localhost', 80
@@ -1241,6 +1591,8 @@ class TestNetHttpPersistent < MiniTest::Unit::TestCase
   end
 
   def test_ssl_verify_mode
+    skip 'OpenSSL is missing' unless HAVE_OPENSSL
+
     @http.verify_mode = OpenSSL::SSL::VERIFY_NONE
     c = Net::HTTP.new 'localhost', 80
 
@@ -1251,35 +1603,41 @@ class TestNetHttpPersistent < MiniTest::Unit::TestCase
   end
 
   def test_ssl_warning
-    orig_verify_peer = OpenSSL::SSL::VERIFY_PEER
-    OpenSSL::SSL.send :remove_const, :VERIFY_PEER
-    OpenSSL::SSL.send :const_set, :VERIFY_PEER, OpenSSL::SSL::VERIFY_NONE
+    skip 'OpenSSL is missing' unless HAVE_OPENSSL
 
-    c = Net::HTTP.new 'localhost', 80
+    begin
+      orig_verify_peer = OpenSSL::SSL::VERIFY_PEER
+      OpenSSL::SSL.send :remove_const, :VERIFY_PEER
+      OpenSSL::SSL.send :const_set, :VERIFY_PEER, OpenSSL::SSL::VERIFY_NONE
 
-    out, err = capture_io do
-      @http.ssl c
-    end
+      c = Net::HTTP.new 'localhost', 80
 
-    assert_empty out
+      out, err = capture_io do
+        @http.ssl c
+      end
 
-    assert_match %r%localhost:80%, err
-    assert_match %r%I_KNOW_THAT_OPENSSL%, err
+      assert_empty out
 
-    Object.send :const_set, :I_KNOW_THAT_OPENSSL_VERIFY_PEER_EQUALS_VERIFY_NONE_IS_WRONG, nil
+      assert_match %r%localhost:80%, err
+      assert_match %r%I_KNOW_THAT_OPENSSL%, err
 
-    assert_silent do
-      @http.ssl c
-    end
-  ensure
-    OpenSSL::SSL.send :remove_const, :VERIFY_PEER
-    OpenSSL::SSL.send :const_set, :VERIFY_PEER, orig_verify_peer
-    if Object.const_defined?(:I_KNOW_THAT_OPENSSL_VERIFY_PEER_EQUALS_VERIFY_NONE_IS_WRONG) then
-      Object.send :remove_const, :I_KNOW_THAT_OPENSSL_VERIFY_PEER_EQUALS_VERIFY_NONE_IS_WRONG
+      Object.send :const_set, :I_KNOW_THAT_OPENSSL_VERIFY_PEER_EQUALS_VERIFY_NONE_IS_WRONG, nil
+
+      assert_silent do
+        @http.ssl c
+      end
+    ensure
+      OpenSSL::SSL.send :remove_const, :VERIFY_PEER
+      OpenSSL::SSL.send :const_set, :VERIFY_PEER, orig_verify_peer
+      if Object.const_defined?(:I_KNOW_THAT_OPENSSL_VERIFY_PEER_EQUALS_VERIFY_NONE_IS_WRONG) then
+        Object.send :remove_const, :I_KNOW_THAT_OPENSSL_VERIFY_PEER_EQUALS_VERIFY_NONE_IS_WRONG
+      end
     end
   end
 
   def test_ssl_cleanup
+    skip 'OpenSSL is missing' unless HAVE_OPENSSL
+
     uri1 = URI.parse 'https://one.example'
 
     c1 = @http.connection_for uri1
@@ -1301,7 +1659,29 @@ class TestNetHttpPersistent < MiniTest::Unit::TestCase
 
     assert_equal :ssl_version, @http.ssl_version
     assert_equal 1, @http.ssl_generation
-  end if RUBY_VERSION > '1.9'
+  end unless RUBY_1
+
+  def test_start
+    c = basic_connection
+
+    @http.socket_options << [Socket::SOL_SOCKET, Socket::SO_KEEPALIVE, 1]
+    @http.debug_output = $stderr
+    @http.open_timeout = 6
+
+    @http.start c
+
+    assert_equal $stderr, c.debug_output
+    assert_equal 6,       c.open_timeout
+
+    socket = c.instance_variable_get :@socket
+
+    expected = []
+    expected << [Socket::IPPROTO_TCP, Socket::TCP_NODELAY, 1] if
+      Socket.const_defined? :TCP_NODELAY
+    expected << [Socket::SOL_SOCKET, Socket::SO_KEEPALIVE, 1]
+
+    assert_equal expected, socket.io.instance_variable_get(:@setsockopts)
+  end
 
   def test_verify_callback_equals
     @http.verify_callback = :verify_callback
